#
# Skeleton ZopeTestCase
#

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

#os.environ['STUPID_LOG_FILE'] = os.path.join(os.getcwd(), 'zLOG.log')
#os.environ['STUPID_LOG_SEVERITY'] = '-200'  # DEBUG

from Testing import ZopeTestCase

ZopeTestCase.installProduct('ExtFile')

_standard_permissions = ZopeTestCase._standard_permissions
_access_permissions = ['View management screens']
_copymove_permissions = ['View management screens', 'Add ExtImages', 'Add ExtFiles', 'Delete objects']
_change_permissions = ['Change ExtFile/ExtImage']

from cgi import FieldStorage
from ZPublisher.HTTPRequest import FileUpload

from Products.ExtFile import ExtFile, Config
ExtFile.ExtFile._repository = ['reposit']
ExtFile.REPOSITORY = Config.FLAT
ExtFile.REPOSITORY_EXTENSIONS = Config.MIMETYPE_REPLACE
ExtFile.UNDO_POLICY = Config.BACKUP_ON_DELETE


local_instance_home = os.getcwd() # Remember this on import!

gifImage = '%s/data/Folder_icon.gif' % local_instance_home
jpegImage = '%s/data/Teneriffa_small.jpg' % local_instance_home


class TestCaseWithLocalInstanceHome(ZopeTestCase.ZopeTestCase):

    def afterSetUp(self):
        # Point INSTANCE_HOME to tests directory
        builtins = getattr(__builtins__, '__dict__', __builtins__)
        self._old = builtins.get('INSTANCE_HOME', None)
        builtins['INSTANCE_HOME'] = local_instance_home

    def afterClear(self):
        # Restore INSTANCE_HOME
        if hasattr(self, '_old'):
            builtins = getattr(__builtins__, '__dict__', __builtins__)
            if self._old is None: 
                del builtins['INSTANCE_HOME']
            else: 
                builtins['INSTANCE_HOME'] = self._old
            del self._old
    

class TestBase(TestCaseWithLocalInstanceHome):

    def afterSetUp(self):
        TestCaseWithLocalInstanceHome.afterSetUp(self)
        self._nuke = 1

    def afterClear(self):
        if hasattr(self, '_nuke') and self._nuke:
            # Remove repository
            repository = os.path.join(INSTANCE_HOME, 'reposit')
            if os.path.isdir(repository):
                import shutil
                shutil.rmtree(repository, 1)
            del self._nuke
        TestCaseWithLocalInstanceHome.afterClear(self)
        try: del self.image
        except AttributeError: pass

    def _fsname(self, id):
        return os.path.join(INSTANCE_HOME, 'reposit', id)
        
    def _exists(self, id):
        return os.path.isfile(self._fsname(id))

    def _listdir(self):
        return os.listdir(os.path.join(INSTANCE_HOME, 'reposit'))

    def _fsize(self, id):
        return os.stat(id)[6]

    def addExtFile(self, id, file):
        # Add an ExtFile
        dispatcher = self.folder.manage_addProduct['ExtFile']
        dispatcher.manage_addExtFile(id=id, file=file)
        self.failUnless(hasattr(self.folder, id))
        self.image = self.folder[id]

    def addExtImage(self, id, file):
        # Add an ExtImage
        dispatcher = self.folder.manage_addProduct['ExtFile']
        dispatcher.manage_addExtImage(id=id, file=file)
        self.failUnless(hasattr(self.folder, id))
        self.image = self.folder[id]


class TestExtFileAdd(TestBase):
    '''Test ExtFile/ExtImage creation and content-type detection'''

    def testAddImage1(self):
        '''Add Image from file name'''
        self.addExtImage(id='image', file=gifImage)
        assert self.image.get_size() == self._fsize(gifImage)

    def testAddImage2(self):
        '''Add Image from file handle'''
        self.addExtImage(id='image', file=open(gifImage, 'rb'))
        assert self.image.get_size() == self._fsize(gifImage)

    def testAddImage3(self):
        '''Add Image from file upload'''
        environ = {'REQUEST_METHOD': 'POST', 'CONTENT_TYPE': 'image/gif'}
        file = FileUpload(FieldStorage(fp=open(gifImage, 'rb'), environ=environ))
        self.addExtImage(id='image', file=file)
        assert self.image.get_size() == self._fsize(gifImage)

    def testContentType1(self):
        '''Detect GIF content type from file extension'''
        self.addExtFile(id='image.gif', file=gifImage)
        assert self.image.content_type == 'image/gif', 'Content type: ' + self.image.content_type
            
    def testContentType2(self):
        '''Detect GIF content type from file upload headers'''
        environ = {'REQUEST_METHOD': 'POST', 'CONTENT_TYPE': 'image/gif'}
        file = FileUpload(FieldStorage(fp=open(gifImage, 'rb'), environ=environ))
        self.addExtFile(id='image', file=file)
        assert self.image.content_type == 'image/gif', 'Content type: ' + self.image.content_type

    def testContentType3(self):
        '''Detect GIF content type from file contents'''
        self.addExtImage(id='image', file=gifImage)
        assert self.image.content_type == 'image/gif', 'Content type: ' + self.image.content_type
            
    def testContentType4(self):
        '''Detect GIF content type from string buffer'''
        self.addExtImage(id='image', file=jpegImage)
        self.image.manage_upload(file=open(gifImage, 'rb').read())
        assert self.image.content_type == 'image/gif', 'Content type: ' + self.image.content_type
        # check manage_upload works
        assert self.image.get_size() == self._fsize(gifImage)

    def testContentType5(self):
        '''Detect JPEG content type from file contents'''
        self.addExtImage(id='image', file=jpegImage)
        assert self.image.content_type == 'image/jpeg', 'Content type: ' + self.image.content_type

    def testContentType6(self):
        '''Detect JPEG content type from PUT' image'''
        self.addExtImage(id='image', file=open(gifImage, 'rb'))
        request = self.app.REQUEST
        request['BODYFILE'] = open(jpegImage, 'rb')
        self.image.PUT(request, request.RESPONSE)
        assert self.image.content_type == 'image/jpeg'
        # check PUT works
        assert self.image.get_size() == self._fsize(jpegImage)


class TestExtFileRepository(TestBase):
    '''Test file repository handling'''

    def afterSetUp(self):
        TestBase.afterSetUp(self)
        self.folder.manage_addFolder('subfolder')
        self.subfolder = self.folder['subfolder']

    def afterClear(self):
        TestBase.afterClear(self)
        try: del self.subfolder
        except AttributeError: pass

    def testName1(self):
        '''Create a unique file name'''
        self.addExtImage(id='image', file=gifImage)
        dispatcher = self.subfolder.manage_addProduct['ExtFile']
        dispatcher.manage_addExtImage(id='image', file=gifImage)
        self.failUnless(self._exists('image.gif'))
        self.failIf(self._exists('image.gif.undo'))
        self.failUnless(self._exists('image1.gif'))
        self.failIf(self._exists('image1.gif.undo'))
         
    def testName2(self):
        '''Create a unique preview name'''
        self.addExtImage(id='image', file=jpegImage)
        self.image.manage_create_prev(100, 100, ratio=1)
        self.failUnless(self._exists('image.jpg'))
        self.failIf(self._exists('image.jpg.undo'))
        self.failUnless(self._exists('image1.jpg'))
        self.failIf(self._exists('image1.jpg.undo'))
         
    def testUndoName1(self):
        '''Create a .undo file'''
        self.addExtImage(id='image', file=gifImage)
        self.folder.manage_delObjects(['image'])
        self.failIf(self._exists('image.gif'))
        self.failUnless(self._exists('image.gif.undo'))
         
    def testUndoName2(self):
        '''Do not create a .undo file'''
        self.addExtImage(id='image', file=gifImage)
        self.image.manage_file_upload(gifImage)
        self.failUnless(self._exists('image.gif'))
        self.failIf(self._exists('image.gif.undo'))
        self.failIf(self._exists('image1.gif'))


class TestExtFileCopyPaste(TestBase):
    '''Tests cut/copy/paste/rename/clone'''

    def afterSetUp(self):
        TestBase.afterSetUp(self)
        self.folder.manage_addFolder('subfolder')
        self.subfolder = self.folder['subfolder']
        self.setPermissions(_standard_permissions+_copymove_permissions)
        self.addExtFile(id='image.gif', file=gifImage)
        get_transaction().commit(1) # need a _p_jar

    def afterClear(self):
        TestBase.afterClear(self)
        try: del self.subfolder
        except AttributeError: pass

    def testClone(self):
        '''Clone a file'''
        #print self._listdir()
        self.subfolder.manage_clone(self.image, 'image.gif')
        #print self._listdir()
        self.failUnless(self._exists('image.gif'))
        self.failIf(self._exists('image.gif.undo'))
        # This only works in Zope 2.6+
        self.failUnless(self._exists('image1.gif'))
        #print self._listdir()
        #print self.image.get_filename()
        #print self.subfolder.image.get_filename()

    def testCopyPaste(self):
        '''Copy and paste a file'''
        cb = self.folder.manage_copyObjects(['image.gif'])
        self.subfolder.manage_pasteObjects(cb)
        self.failUnless(self._exists('image.gif'))
        self.failIf(self._exists('image.gif.undo'))
        self.failUnless(self._exists('image1.gif'))
        assert self.subfolder['image.gif'].get_size() == self._fsize(gifImage)
         
    def testCutPaste(self):
        '''Cut and paste a file'''
        cb = self.folder.manage_cutObjects(['image.gif'])
        self.subfolder.manage_pasteObjects(cb)
        self.failUnless(self._exists('image.gif'))
        self.failIf(self._exists('image.gif.undo'))
        self.failIf(self._exists('image1.gif'))

    def testRename(self):
        '''Rename a file'''
        self.folder.manage_renameObject('image.gif', 'image44.gif')
        self.failUnless(self._exists('image.gif'))
        self.failIf(self._exists('image.gif.undo'))
        self.failIf(self._exists('image1.gif'))

    def testCOPY(self):
        '''Webdav copy a file'''
        req = self.app.REQUEST
        req.environ['HTTP_DEPTH'] = 'infinity'
        req.environ['HTTP_DESTINATION'] = 'http://dummy.org/%s/subfolder/image.gif' %ZopeTestCase._folder_name
        self.folder['image.gif'].COPY(req, req.RESPONSE)
        self.failUnless(self._exists('image.gif'))
        self.failIf(self._exists('image.gif.undo'))
        self.failUnless(self._exists('image1.gif'))
        assert self.subfolder['image.gif'].get_size() == self._fsize(gifImage)


class TestExtImageCopyPaste(TestBase):
    '''Tests cut/copy/paste/rename/clone'''

    def afterSetUp(self):
        TestBase.afterSetUp(self)
        self.folder.manage_addFolder('subfolder')
        self.subfolder = self.folder['subfolder']
        self.setPermissions(_standard_permissions+_copymove_permissions)
        self.addExtImage(id='image.gif', file=gifImage)
        # Add a preview image as well
        self.folder['image.gif'].manage_file_upload(file=gifImage, is_preview=1)
        get_transaction().commit(1) # need a _p_jar

    def afterClear(self):
        TestBase.afterClear(self)
        try: del self.subfolder
        except AttributeError: pass

    def testClone(self):
        '''Clone an image'''
        self.subfolder.manage_clone(self.image, 'image.gif')
        self.failUnless(self._exists('image.gif'))
        self.failIf(self._exists('image.gif.undo'))
        self.failUnless(self._exists('image1.gif'))
        self.failIf(self._exists('image1.gif.undo'))
        # This only works in Zope 2.6+
        self.failUnless(self._exists('image2.gif'))
        self.failUnless(self._exists('image3.gif'))

    def testCopyPaste(self):
        '''Copy and paste an image'''
        cb = self.folder.manage_copyObjects(['image.gif'])
        self.subfolder.manage_pasteObjects(cb)
        self.failUnless(self._exists('image.gif'))
        self.failIf(self._exists('image.gif.undo'))
        self.failUnless(self._exists('image1.gif'))
        self.failIf(self._exists('image1.gif.undo'))
        self.failUnless(self._exists('image2.gif'))
        self.failUnless(self._exists('image3.gif'))
        assert self.subfolder['image.gif'].get_size() == self._fsize(gifImage)
         
    def testCutPaste(self):
        '''Cut and paste an image'''
        cb = self.folder.manage_cutObjects(['image.gif'])
        self.subfolder.manage_pasteObjects(cb)
        self.failUnless(self._exists('image.gif'))
        self.failIf(self._exists('image.gif.undo'))
        self.failUnless(self._exists('image1.gif'))
        self.failIf(self._exists('image1.gif.undo'))
        self.failIf(self._exists('image2.gif'))
        self.failIf(self._exists('image3.gif'))

    def testRename(self):
        '''Rename an image'''
        self.folder.manage_renameObject('image.gif', 'image44.gif')
        self.failUnless(self._exists('image.gif'))
        self.failIf(self._exists('image.gif.undo'))
        self.failUnless(self._exists('image1.gif'))
        self.failIf(self._exists('image1.gif.undo'))
        self.failIf(self._exists('image2.gif'))
        self.failIf(self._exists('image3.gif'))

    def testCOPY(self):
        '''Webdav copy an image'''
        req = self.app.REQUEST
        req.environ['HTTP_DEPTH'] = 'infinity'
        req.environ['HTTP_DESTINATION'] = 'http://dummy.org/%s/subfolder/image.gif' %ZopeTestCase._folder_name
        self.folder['image.gif'].COPY(req, req.RESPONSE)
        self.failUnless(self._exists('image.gif'))
        self.failIf(self._exists('image.gif.undo'))
        self.failUnless(self._exists('image1.gif'))
        self.failIf(self._exists('image1.gif.undo'))
        self.failUnless(self._exists('image2.gif'))
        self.failUnless(self._exists('image3.gif'))
        assert self.subfolder['image.gif'].get_size() == self._fsize(gifImage)


if __name__ == '__main__':
    framework(descriptions=1, verbosity=1)
else:
    import unittest
    def test_suite():
        suite = unittest.TestSuite()
        suite.addTest(unittest.makeSuite(TestExtFileAdd))
        suite.addTest(unittest.makeSuite(TestExtFileRepository))
        suite.addTest(unittest.makeSuite(TestExtFileCopyPaste))
        suite.addTest(unittest.makeSuite(TestExtImageCopyPaste))
        return suite

